/* UART routines
 */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <setjmp.h>
#include "sig455.h"
#include "uart.h"

ISR(USART_RX_vect) {
	longjmp(jmpb, 1);
}

void 
uart_init(void) {
	/* set baudrate */
	UBRR0 = F_CPU /16 /BAUDRATE -1;
	/* enable transmitter */
	UCSR0B |= _BV(TXEN0) | _BV(RXEN0) | _BV(RXCIE0);
	//UCSR0B |= _BV(TXEN0) | _BV(RXEN0);
}

int8_t
uart_getc(void)
{
	/* wait till data available */
	while ( !(UCSR0A & _BV(RXC0)) )
	        ;
	return UDR0;
}

void
uart_putc(int8_t data) {
	/* wait till data port empty */
	while (! (UCSR0A & _BV(UDRE0)))
	        ;
	UDR0 = data;
}

void
uart_puts(char *s)
{
	while (*s)
	        uart_putc(*s++);
}

char *hexchars = "0123456789abcdef";
void
uart_hexbyte(int8_t data) {
	uart_putc(hexchars[(data>>4) & 0xf]);
	uart_putc(hexchars[data & 0xf]);
}

void
uart_hexword(int w)
{
	uart_hexbyte( (w >> 8) & 255);
	uart_hexbyte( w & 255);
}

void
uart_hexlong(long w)
{
	uart_hexword( (w >> 16) & 65535);
	uart_hexword( w & 65535);
}

